@@ -2,6 +2,7 @@ import time
 import threading
 import weakref
 from lru.heap import Heap
+from lru.utils import BypassThreadSafe
 from abc import ABC, abstractmethod, ABCMeta
 
 class BoundedLRUCache:
@@ -14,7 +15,7 @@ class BoundedLRUCache:
 
 class LRUCache(BoundedLRUCache):
 
-    def __init__(self, capacity: int=128, seconds: int=60*15):
+    def __init__(self, capacity: int=128, seconds: int=60*15, thread_safe: bool=False):
         """Constructor for LRU Cache objects. Given
         the several parameter such as:
 
@@ -25,14 +26,31 @@ class LRUCache(BoundedLRUCache):
         maximum is 15 minutes
         
         :dict: param for set the key and value into dict type
+
+        :thread_safe: param for enable/disable thread safe option,
+        default is False
         """
         self.capacity = capacity
         self.seconds = seconds
         self.dict = {}
         self.cache = Heap()
+        self.lock = threading.RLock() if thread_safe else BypassThreadSafe()
 
-        if capacity is None:
-            raise ValueError("Capacity must be set.")
+        # user type checking
+        # will bump into latest version -> 0.1.3
+        if not isinstance(capacity, int):
+            raise ValueError("Expected to set the capacity of cache.")
+        elif capacity > 128:
+            raise ValueError("Maximum capacity of cache is 128.")
+        elif capacity < 0:
+            raise ValueError("Cache capacity can't set below zero.")
+
+        if not isinstance(seconds, int):
+            raise ValueError("Expected to set the duration of cache in seconds time.")
+        elif seconds > 60*15:
+            raise ValueError("Maximum duration of cache is 15 minutes.")
+        elif seconds < 0:
+            raise ValueError("Cache duration can't set below zero time.")
 
     def __call__(self, *args, **kwargs):
         return self.cache
@@ -40,12 +58,22 @@ class LRUCache(BoundedLRUCache):
     def __len__(self):
         return len(self.capacity)
 
+    def __str__(self):
+        return "%s" % self.dict
+
+    def is_empty(self):
+        if len(self.dict) == 0:
+            return True
+        return False
+
     def clear_all(self):
         return self.dict.clear()
 
     def clear_cache_key(self, key: int) -> int:
         """Clear cache in element based on their key."""
-        return self.cache.remove_key(key)
+        with self.lock:
+            if self.get_cache(key):
+                return self.dict.clear()
 
     def get_duration(self, expired_time: int=3600) -> int:
         """Get duration of cache, return `True` if the duration
@@ -73,9 +101,10 @@ class LRUCache(BoundedLRUCache):
         `True` if the element has a key, otherwise return `False`
         when element hasn't a key.
         """
-        if self.dict.get(key):
-            return True
-        return False
+        with self.lock:
+            if self.dict.get(key):
+                return True
+            return False
 
     def get_capacity(self):
         """Get cache capacity, return `True` if the cache
@@ -121,6 +150,9 @@ class LRUCache(BoundedLRUCache):
         if not self.get_cache(key):
             raise KeyError("Cache key not in elements.")
 
+        if self.is_empty:
+            raise KeyError("Nor between cache key and cache is empty.")
+
         access_time = time.perf_counter()
         self.cache.update(key, access_time)
         value = self.dict.get(key)[0]
@@ -129,68 +161,25 @@ class LRUCache(BoundedLRUCache):
 
     def get_lru_element(self):
         """Returned a dict type based on their key in cache element."""
-        key = self.cache.heap[0][1]
-        return self.dict[key]
+        with self.lock:
+            key = self.cache.heap[0][1]
+            return self.dict[key]
 
     def get_dict(self):
         """Returned a dict type in cache element."""
         return self.dict
 
-class ThreadSafe(LRUCache):
-    """Inheritance classes from LRU Cache for enable
-    thread safe parameter. Whenever this class is used
-    for the cached object, the global environment will applies
-    multi-threaded code and make sure that LRUCache method
-    works properly.
-    """
-    def __init__(self, capacity: int=128, seconds: int=60*15, thread_safe=False):
-        # super().__init__(capacity=capacity, seconds=seconds)
-        self.capacity = capacity
-        self.seconds = seconds
-        self.thread_safe = thread_safe
-        self.dict = Heap()
-
-        if thread_safe:
-            thread_safe = self.EmptyCache(threading.Thread)
-            thread_safe.start()
-
-    class EmptyCache(threading.Thread):
-        # thread same as the daemon
-        thread = True
-
-        def __init__(self, cache: int, duration: int=60):
-            # thread must be terminate
-            # if thread not terminat, 
-            # it will be looping over again
-            # for right now using this method
-            # to terminate thread
-            def kill_self(self):
-                del self
-
-            self.cache_reference = weakref.ref(cache)
-            self.duration = duration
-            super(ThreadSafe.EmptyCache, self).__init__()
-
-        def start_thread_safe(self):
-            while self.cache_reference():
-                _get_cache = self.cache_reference
-                if _get_cache:
-                    next_cache_expired = _get_cache.cleanup()
-                    if next_cache_expired is None:
-                        time.sleep(self.duration)
-                    else:
-                        time.sleep(next_cache_expired + 1)
-                _get_cache = None
-
 # mock test
 """
-test = LRUCache(128, 60, False)
+test = LRUCache(128, 60, thread_safe=True)
 
 test.set(1, "value")
-test.get(1)
-
-# print(test.get_ttl(1))
-# print(test.clear_cache(1))
 # print(test.clear_all())
-# print(test.get_dict())
+#test.get(1)
+print(test.is_empty())
+print(test.get_dict())
+# print(test.get_ttl(1))
+print(test.clear_cache_key(1))
+# print(test.clear_all())
+print(test.is_empty())
 """
\ No newline at end of file
